@@ -3,7 +3,7 @@
   JackTrip: A System for High-Quality Audio Network Performance
   over the Internet
 
-  Copyright (c) 2022-2024 JackTrip Labs, Inc.
+  Copyright (c) 2022-2025 JackTrip Labs, Inc.
 
   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
@@ -44,6 +44,7 @@
 #include <QFile>
 #include <QFontDatabase>
 #include <QMessageBox>
+#include <QNetworkCookie>
 #include <QQmlContext>
 #include <QQmlEngine>
 #include <QQuickStyle>
@@ -54,7 +55,10 @@
 #include <QStandardPaths>
 #include <QSysInfo>
 #include <QTextStream>
+#include <QWebEngineCookieStore>
+#include <QWebEngineProfile>
 #include <QtGlobal>
+#include <QtWebEngineQuick/QQuickWebEngineProfile>
 #include <algorithm>
 #include <iostream>
 
@@ -135,19 +139,19 @@ VirtualStudio::VirtualStudio(UserInterface& parent)
             &VirtualStudio::slotAuthSucceeded);
     connect(m_auth.data(), &VsAuth::updatedAccessToken, this,
             &VirtualStudio::slotAccessTokenUpdated);
-    connect(m_auth.data(), &VsAuth::refreshTokenFailed, this, [=]() {
+    connect(m_auth.data(), &VsAuth::refreshTokenFailed, this, [this]() {
         m_auth->authenticate(QStringLiteral(""));  // retry without using refresh token
     });
-    connect(m_auth.data(), &VsAuth::fetchUserInfoFailed, this, [=]() {
+    connect(m_auth.data(), &VsAuth::fetchUserInfoFailed, this, [this]() {
         m_auth->authenticate(QStringLiteral(""));  // retry without using refresh token
     });
-    connect(m_auth.data(), &VsAuth::deviceCodeExpired, this, [=]() {
+    connect(m_auth.data(), &VsAuth::deviceCodeExpired, this, [this]() {
         m_auth->authenticate(QStringLiteral(""));  // retry without using refresh token
     });
 
     m_webChannelServer.reset(new QWebSocketServer(
         QStringLiteral("Qt6 Virtual Studio Server"), QWebSocketServer::NonSecureMode));
-    connect(m_webChannelServer.data(), &QWebSocketServer::newConnection, this, [=]() {
+    connect(m_webChannelServer.data(), &QWebSocketServer::newConnection, this, [this]() {
         m_webChannel->connectTo(
             new WebSocketTransport(m_webChannelServer->nextPendingConnection()));
     });
@@ -181,7 +185,7 @@ VirtualStudio::VirtualStudio(UserInterface& parent)
 
     // on window focus, attempt to refresh the access token if the token is more than 1
     // hour old
-    connect(m_view.data(), &VsQuickView::focusGained, this, [=]() {
+    connect(m_view.data(), &VsQuickView::focusGained, this, [this]() {
         QString refreshToken = m_auth->refreshToken();
         if (refreshToken.isEmpty()) {
             return;
@@ -290,16 +294,27 @@ VirtualStudio::VirtualStudio(UserInterface& parent)
 
     // prepare handler for local socket connections
     m_socketServerPtr.reset(new SocketServer());
-    m_socketServerPtr->addHandler("deeplink", [=](QSharedPointer<QLocalSocket>& socket) {
-        m_deepLinkPtr->handleVsDeeplinkRequest(socket);
-    });
-    m_socketServerPtr->addHandler("audio", [=](QSharedPointer<QLocalSocket>& socket) {
+    m_socketServerPtr->addHandler("deeplink",
+                                  [this](QSharedPointer<QLocalSocket>& socket) {
+                                      m_deepLinkPtr->handleVsDeeplinkRequest(socket);
+                                  });
+    m_socketServerPtr->addHandler("audio", [this](QSharedPointer<QLocalSocket>& socket) {
         this->handleAudioSocketRequest(socket);
     });
     m_socketServerPtr->start();
 
-    // initialize default QtWebEngineProfile
-    m_qwebEngineProfile = QWebEngineProfile::defaultProfile();
+    // initialize default profile for WebEngine
+    QQuickWebEngineProfile* defaultWebEngineProfile =
+        QQuickWebEngineProfile::defaultProfile();
+    defaultWebEngineProfile->setStorageName(QStringLiteral("Default"));
+    defaultWebEngineProfile->setHttpUserAgent(
+        QStringLiteral("JackTrip/%1").arg(versionString()));
+    defaultWebEngineProfile->setCachePath(defaultWebEngineProfile->persistentStoragePath()
+                                          + QStringLiteral("/Cache"));
+    defaultWebEngineProfile->setPersistentCookiesPolicy(
+        QQuickWebEngineProfile::ForcePersistentCookies);
+    defaultWebEngineProfile->setHttpCacheType(QQuickWebEngineProfile::DiskHttpCache);
+    defaultWebEngineProfile->setOffTheRecord(false);
 }
 
 void VirtualStudio::show()
@@ -400,10 +415,10 @@ QString VirtualStudio::copyrightString()
 #endif
 
     result +=
-        "Copyright &copy; 2008-2024 Juan-Pablo Caceres, Chris Chafe, et al. SoundWIRE "
+        "Copyright &copy; 2008-2025 Juan-Pablo Caceres, Chris Chafe, et al. SoundWIRE "
         "group at CCRMA, Stanford University.<br/><br/>\n";
     result +=
-        "Virtual Studio interface and integration Copyright &copy; 2022-2024 JackTrip "
+        "Virtual Studio interface and integration Copyright &copy; 2022-2025 JackTrip "
         "Labs, Inc.<br/><br/>\n";
 
     if (hasClassicMode()) {
@@ -592,7 +607,7 @@ void VirtualStudio::setWindowState(QString state)
     m_windowState = state;
     // refresh studio list if navigating to browse window
     // only if user id is empty (edge case for when logging in)
-    if (m_windowState == "browse" && !m_userId.isEmpty()) {
+    if (m_windowState == "browse" && m_auth->isAuthenticated()) {
         // schedule studio refresh instead of doing it now
         // just to reduce risk of running into a deadlock
         emit scheduleStudioRefresh(-1, false);
@@ -655,26 +670,22 @@ void VirtualStudio::collectFeedbackSurvey(QString serverId, int rating, QString
 #endif
 
     feedback.insert(QStringLiteral("osVersion"), QSysInfo::prettyProductName());
-    QString sysInfo = QString("[platform=%1").arg(QSysInfo::prettyProductName());
 #ifdef RT_AUDIO
     QString inputDevice =
         QString::fromStdString(m_audioConfigPtr->getInputDevice().toStdString());
     if (!inputDevice.isEmpty()) {
-        sysInfo.append(QString(",input=%1").arg(inputDevice));
+        feedback.insert(QStringLiteral("inputDevice"), inputDevice);
     }
     QString outputDevice =
         QString::fromStdString(m_audioConfigPtr->getOutputDevice().toStdString());
     if (!outputDevice.isEmpty()) {
-        sysInfo.append(QString(",output=%1").arg(outputDevice));
+        feedback.insert(QStringLiteral("outputDevice"), outputDevice);
     }
 #endif
-    sysInfo.append("]");
 
     feedback.insert(QStringLiteral("rating"), rating);
-    if (message.isEmpty()) {
-        feedback.insert(QStringLiteral("message"), sysInfo);
-    } else {
-        feedback.insert(QStringLiteral("message"), message + " " + sysInfo);
+    if (!message.isEmpty()) {
+        feedback.insert(QStringLiteral("message"), message);
     }
 
     QString deviceIssues  = "";
@@ -687,11 +698,11 @@ void VirtualStudio::collectFeedbackSurvey(QString serverId, int rating, QString
     }
     if (!deviceIssues.isEmpty()) {
         feedback.insert(QStringLiteral("deviceIssues"), deviceIssues);
-        message.append(" (deviceIssues=" + deviceIssues + ")");
     }
 
     QJsonDocument data = QJsonDocument(feedback);
     m_api->submitServerFeedback(serverId, data.toJson());
+    std::cout << "Sent feedback: " << data.toJson().toStdString() << std::endl;
     return;
 }
 
@@ -857,17 +868,13 @@ void VirtualStudio::joinStudio()
         return;
     }
 
-    // pop studioToJoin
-    const QString targetId = m_studioToJoin;
-    setStudioToJoin("");
-
     // stop audio if already running (settings or setup windows)
     m_audioConfigPtr->stopAudio(true);
 
     // find and populate data for current studio
     VsServerInfoPointer sPtr;
     for (const VsServerInfoPointer& s : m_servers) {
-        if (s->id() == targetId) {
+        if (s->id() == m_studioToJoin) {
             sPtr = s;
             break;
         }
@@ -875,21 +882,23 @@ void VirtualStudio::joinStudio()
     locker.unlock();
 
     if (sPtr.isNull()) {
-        m_failedMessage = "Unable to find studio " + targetId;
+        m_failedMessage = "Unable to find studio " + m_studioToJoin;
+        setStudioToJoin("");
         emit failedMessageChanged();
         emit failed();
         return;
     }
 
-    m_currentStudio = *sPtr;
-    emit currentStudioChanged();
-
     if (m_windowState == "setup") {
-        m_audioConfigPtr->setSampleRate(m_currentStudio.sampleRate());
+        m_audioConfigPtr->setSampleRate(sPtr->sampleRate());
         m_audioConfigPtr->startAudio();
         return;
     }
 
+    setStudioToJoin("");
+    m_currentStudio = *sPtr;
+    emit currentStudioChanged();
+
     // m_windowState == "connected"
     connectToStudio();
 }
@@ -1168,6 +1177,9 @@ void VirtualStudio::triggerReconnect(bool refresh)
         return;
     }
 
+    std::cout << "Reconnecting audio to " << m_currentStudio.host().toStdString() << ":"
+              << m_currentStudio.port() << std::endl;
+
     // this needs to be synchronous to avoid both trying
     // to use the audio interfaces at the same time
     // note that connectionFinished() checks m_reconnectState
@@ -1319,6 +1331,13 @@ void VirtualStudio::handleAudioSocketRequest(QSharedPointer<QLocalSocket>& socke
 {
     QSharedPointer<AudioSocket> audioSocketPtr(new AudioSocket(socket));
     m_audioConfigPtr->registerAudioSocket(audioSocketPtr);
+    if (!m_jackTripRunning || m_devicePtr.isNull()) {
+        if (m_audioConfigPtr->getAudioReady()) {
+            // no need to refresh or validate devices, just restart audio
+            m_audioConfigPtr->restartAudio();
+        }
+        return;
+    }
     triggerReconnect(true);
 }
 
@@ -1390,24 +1409,14 @@ void VirtualStudio::slotAuthSucceeded()
 
 void VirtualStudio::slotAccessTokenUpdated(QString accessToken)
 {
-    // set cookie
-    QWebEngineCookieStore* cookieStore = m_qwebEngineProfile->cookieStore();
-    QNetworkCookie authCookie =
-        QNetworkCookie(QByteArray("auth_code"), accessToken.toUtf8());
+    // set auth cookies for prod mode
+    setCookie("auth_code", accessToken, "https://www.jacktrip.com");
+    setCookie("auth_code", accessToken, "https://app.jacktrip.com");
 
-    QUrl url1 = QUrl(QStringLiteral("https://www.jacktrip.com"));
-    QUrl url2 = QUrl(QStringLiteral("https://app.jacktrip.com"));
-    QUrl url3 = QUrl(QStringLiteral("http://localhost:3000"));
-    if (testMode()) {
-        url1 = QUrl(QStringLiteral("https://next-test.jacktrip.com"));
-        url2 = QUrl(QStringLiteral("https://test.jacktrip.com"));
-    }
-
-    cookieStore->setCookie(authCookie, url1);
-    cookieStore->setCookie(authCookie, url2);
-    if (testMode()) {
-        cookieStore->setCookie(authCookie, url3);
-    }
+    // set auth cookies for test mode
+    setCookie("auth_code", accessToken, "https://next-test.jacktrip.com");
+    setCookie("auth_code", accessToken, "https://test.jacktrip.com");
+    setCookie("auth_code", accessToken, "http://localhost:3000");
 
     // Get refresh token and userId
     m_refreshToken = m_auth->refreshToken();
@@ -1420,6 +1429,16 @@ void VirtualStudio::slotAccessTokenUpdated(QString accessToken)
     settings.endGroup();
 }
 
+void VirtualStudio::setCookie(const QString& name, const QString& value,
+                              const QString& origin)
+{
+    // set webengine cookie
+    QNetworkCookie cookie = QNetworkCookie(name.toUtf8(), value.toUtf8());
+    QWebEngineCookieStore* cookieStore =
+        QWebEngineProfile::defaultProfile()->cookieStore();
+    cookieStore->setCookie(cookie, origin);
+}
+
 void VirtualStudio::connectionFinished()
 {
     if (!m_devicePtr.isNull()
@@ -1444,8 +1463,10 @@ void VirtualStudio::connectionFinished()
 void VirtualStudio::processError(const QString& errorMessage)
 {
     static const QString RtAudioErrorMsg = QStringLiteral("RtAudio Error");
+    static const QString RtApiErrorMsg   = QStringLiteral("RtApiCore: ");
     static const QString JackAudioErrorMsg =
         QStringLiteral("The Jack server was shut down");
+    static const QString DisconnectedErrorMsg = QStringLiteral("device was disconnected");
 
     const bool shouldSwitchToRtAudio =
         (errorMessage == QLatin1String("Maybe the JACK server is not running?"));
@@ -1462,7 +1483,7 @@ void VirtualStudio::processError(const QString& errorMessage)
     } else if (errorMessage.startsWith(RtAudioErrorMsg)) {
         if (errorMessage.length() > RtAudioErrorMsg.length() + 2) {
             const QString details(errorMessage.sliced(RtAudioErrorMsg.length() + 2));
-            if (details.contains(QStringLiteral("device was disconnected"))
+            if (details.contains(DisconnectedErrorMsg)
                 || details.contains(
                     QStringLiteral("Unable to retrieve capture buffer"))) {
                 msgBox.setText(QStringLiteral("Your audio interface was disconnected."));
@@ -1480,6 +1501,13 @@ void VirtualStudio::processError(const QString& errorMessage)
             msgBox.setText(QStringLiteral("The JACK Audio Server was stopped."));
         }
         msgBox.setWindowTitle(QStringLiteral("Jack Audio Error"));
+    } else if (errorMessage.startsWith(RtApiErrorMsg)) {
+        if (errorMessage.contains(DisconnectedErrorMsg)) {
+            msgBox.setText(QStringLiteral("Your audio interface was disconnected."));
+        } else {
+            msgBox.setText(errorMessage.sliced(RtApiErrorMsg.length()));
+        }
+        msgBox.setWindowTitle(QStringLiteral("Audio Interface Error"));
     } else {
         msgBox.setText(QStringLiteral("Error: ").append(errorMessage));
         msgBox.setWindowTitle(QStringLiteral("Doh!"));
@@ -1501,11 +1529,18 @@ void VirtualStudio::receivedConnectionFromPeer()
 
 void VirtualStudio::handleWebsocketMessage(const QString& msg)
 {
-    QJsonObject serverState = QJsonDocument::fromJson(msg.toUtf8()).object();
-    QString serverStatus    = serverState[QStringLiteral("status")].toString();
-    bool serverEnabled      = serverState[QStringLiteral("enabled")].toBool();
-    QString serverCloudId   = serverState[QStringLiteral("cloudId")].toString();
-    int queueBuffer         = serverState[QStringLiteral("queueBuffer")].toInt();
+    if (m_currentStudio.id() == "") {
+        return;
+    }
+
+    QJsonObject serverState      = QJsonDocument::fromJson(msg.toUtf8()).object();
+    const QString& serverHost    = serverState[QStringLiteral("serverHost")].toString();
+    const QString& serverStatus  = serverState[QStringLiteral("status")].toString();
+    const QString& serverCloudId = serverState[QStringLiteral("cloudId")].toString();
+    const QString& sessionId     = serverState[QStringLiteral("sessionId")].toString();
+    const bool serverEnabled     = serverState[QStringLiteral("enabled")].toBool();
+    const int serverPort         = serverState[QStringLiteral("serverPort")].toInt();
+    const int queueBuffer        = serverState[QStringLiteral("queueBuffer")].toInt();
 
     // server notifications are also transmitted along this websocket, so ignore data if
     // it contains "message"
@@ -1513,26 +1548,52 @@ void VirtualStudio::handleWebsocketMessage(const QString& msg)
     if (!message.isEmpty()) {
         return;
     }
-    if (m_currentStudio.id() == "") {
-        return;
+
+    bool currentStudioUpdated = false;
+    if (serverHost != m_currentStudio.host()) {
+        m_currentStudio.setHost(serverHost);
+        currentStudioUpdated = true;
     }
-    m_currentStudio.setStatus(serverStatus);
-    m_currentStudio.setEnabled(serverEnabled);
-    m_currentStudio.setCloudId(serverCloudId);
-    m_currentStudio.setQueueBuffer(queueBuffer);
-    if (!m_jackTripRunning) {
-        if (serverStatus == QLatin1String("Ready") && m_onConnectedScreen) {
-            m_currentStudio.setHost(serverState[QStringLiteral("serverHost")].toString());
-            m_currentStudio.setPort(serverState[QStringLiteral("serverPort")].toInt());
-            m_currentStudio.setSessionId(
-                serverState[QStringLiteral("sessionId")].toString());
-            completeConnection();
+    if (serverStatus != m_currentStudio.status()) {
+        m_currentStudio.setStatus(serverStatus);
+        currentStudioUpdated = true;
+    }
+    if (serverCloudId != m_currentStudio.cloudId()) {
+        m_currentStudio.setCloudId(serverCloudId);
+        currentStudioUpdated = true;
+    }
+    if (sessionId != m_currentStudio.sessionId()) {
+        m_currentStudio.setSessionId(sessionId);
+        currentStudioUpdated = true;
+    }
+    if (serverEnabled != m_currentStudio.enabled()) {
+        m_currentStudio.setEnabled(serverEnabled);
+        currentStudioUpdated = true;
+    }
+    if (serverPort != m_currentStudio.port()) {
+        m_currentStudio.setPort(serverPort);
+        currentStudioUpdated = true;
+    }
+    if (queueBuffer != m_currentStudio.queueBuffer()) {
+        m_currentStudio.setQueueBuffer(queueBuffer);
+        currentStudioUpdated = true;
+        if (m_useStudioQueueBuffer && !m_devicePtr.isNull()) {
+            m_devicePtr->setQueueBuffer(m_currentStudio.queueBuffer());
         }
-    } else if (m_useStudioQueueBuffer && !m_devicePtr.isNull()) {
-        m_devicePtr->setQueueBuffer(m_currentStudio.queueBuffer());
     }
 
-    emit currentStudioChanged();
+    if (currentStudioUpdated) {
+        emit currentStudioChanged();
+    }
+
+    if (m_onConnectedScreen) {
+        if (!m_jackTripRunning && serverEnabled && serverStatus == QLatin1String("Ready")
+            && serverHost != "" && serverPort != 0) {
+            std::cout << "Connecting audio to " << serverHost.toStdString() << ":"
+                      << serverPort << std::endl;
+            completeConnection();
+        }
+    }
 }
 
 void VirtualStudio::restartStudioSocket()
@@ -1586,7 +1647,7 @@ void VirtualStudio::resetState()
 void VirtualStudio::refreshStudios(int index, bool signalRefresh)
 {
     // user id is required for retrieval of subscriptions
-    if (m_userId.isEmpty()) {
+    if (!m_auth->isAuthenticated()) {
         std::cerr << "Studio refresh cancelled due to empty user id" << std::endl;
         return;
     }
@@ -1856,23 +1917,30 @@ void VirtualStudio::detectedFeedbackLoop()
 VirtualStudio::~VirtualStudio()
 {
     QDesktopServices::unsetUrlHandler("jacktrip");
+
     // close the window
     m_view.reset();
+
     // stop the audio worker thread before destructing other things
-    if (!m_audioConfigPtr.isNull()) {
-        m_audioConfigPtr->disconnect();
-        m_audioConfigPtr.reset();
-    }
+    m_audioConfigPtr->stopWorker();
+
     // stop device and corresponding threads
     if (!m_devicePtr.isNull()) {
         m_devicePtr->disconnect();
         m_devicePtr.reset();
     }
+
+    // reset VsAudio after VsDevice since it holds a smart pointer
+    if (!m_audioConfigPtr.isNull()) {
+        m_audioConfigPtr->disconnect();
+        m_audioConfigPtr.reset();
+    }
 }
 
 QApplication* VirtualStudio::createApplication(int& argc, char* argv[])
 {
 #if defined(Q_OS_WIN)
+#if QT_VERSION < QT_VERSION_CHECK(6, 6, 0)
     // Fix for display scaling like 125% or 150% on Windows
     QGuiApplication::setHighDpiScaleFactorRoundingPolicy(
         Qt::HighDpiScaleFactorRoundingPolicy::PassThrough);
@@ -1882,8 +1950,12 @@ QApplication* VirtualStudio::createApplication(int& argc, char* argv[])
     // QCoreApplication::setAttribute(Qt::AA_UseDesktopOpenGL);
     // QCoreApplication::setAttribute(Qt::AA_UseOpenGLES);
 
+    // Direct3D11 is still broken as of Qt 6.8.1
     // QQuickWindow::setGraphicsApi(QSGRendererInterface::Direct3D11);
     QQuickWindow::setGraphicsApi(QSGRendererInterface::OpenGL);
+#else  // Qt 6.6.0 or later supports Direct3D 12, which works well
+    QQuickWindow::setGraphicsApi(QSGRendererInterface::Direct3D12);
+#endif
 #endif
 
     QQuickStyle::setStyle("Basic");
@@ -1891,7 +1963,13 @@ QApplication* VirtualStudio::createApplication(int& argc, char* argv[])
 #if defined(Q_OS_MACOS) && (QT_VERSION > QT_VERSION_CHECK(6, 2, 6)) \
     && (QT_VERSION < QT_VERSION_CHECK(6, 8, 0))
     // work-around for screen sharing bugs in qtwebengine 6.2.7-6.7.x
-    qputenv("QTWEBENGINE_CHROMIUM_FLAGS", "--disable-features=DesktopCaptureMacV2");
+    QString chromiumFlags("--disable-features=DesktopCaptureMacV2");
+    char* existingFlags = getenv("QTWEBENGINE_CHROMIUM_FLAGS");
+    if (existingFlags != nullptr) {
+        chromiumFlags.append(" ");
+        chromiumFlags.append(existingFlags);
+    }
+    qputenv("QTWEBENGINE_CHROMIUM_FLAGS", chromiumFlags.toUtf8());
 #endif
 
     // Initialize webengine
