@@ -3,7 +3,7 @@
   JackTrip: A System for High-Quality Audio Network Performance
   over the Internet
 
-  Copyright (c) 2022-2024 JackTrip Labs, Inc.
+  Copyright (c) 2022-2025 JackTrip Labs, Inc.
 
   Permission is hereby granted, free of charge, to any person
   obtaining a copy of this software and associated documentation
@@ -81,7 +81,7 @@ void VsDevice::registerApp()
 
     // check if device exists
     QNetworkReply* reply = m_api->getDevice(m_appID);
-    connect(reply, &QNetworkReply::finished, this, [=]() {
+    connect(reply, &QNetworkReply::finished, this, [this, reply]() {
         // Got error
         if (reply->error() != QNetworkReply::NoError) {
             QVariant statusCode =
@@ -189,8 +189,17 @@ void VsDevice::sendHeartbeat()
         json.insert(QLatin1String("high_latency"),
                     m_audioConfigPtr->getHighLatencyFlag());
         json.insert(QLatin1String("network_outage"), m_networkOutage);
-        json.insert(QLatin1String("recv_latency"),
-                    m_jackTrip.isNull() ? -1 : m_jackTrip->getLatency());
+        json.insert(QLatin1String("audio_input_latency"),
+                    m_jackTrip.isNull()
+                        ? 0
+                        : (qint64)(m_jackTrip->getAudioInputLatency() * 10000));
+        json.insert(QLatin1String("audio_output_latency"),
+                    m_jackTrip.isNull()
+                        ? 0
+                        : (qint64)(m_jackTrip->getAudioOutputLatency() * 10000));
+        json.insert(
+            QLatin1String("client_buffer_latency"),
+            m_jackTrip.isNull() ? 0 : (qint64)(m_jackTrip->getLatency() * ns_per_ms));
 
         // For the internal application UI, ms will suffice. No conversion needed
         QJsonObject pingStats = {};
@@ -203,8 +212,13 @@ void VsDevice::sendHeartbeat()
                          ((int)(10 * stats.stdDevRtt)) / 10.0);
         pingStats.insert(QLatin1String("highLatency"),
                          m_audioConfigPtr->getHighLatencyFlag());
-        pingStats.insert(QLatin1String("recvLatency"),
-                         m_jackTrip.isNull() ? -1 : m_jackTrip->getLatency());
+        pingStats.insert(QLatin1String("audioInputLatency"),
+                         m_jackTrip.isNull() ? 0 : m_jackTrip->getAudioInputLatency());
+        pingStats.insert(QLatin1String("audioOutputLatency"),
+                         m_jackTrip.isNull() ? 0 : m_jackTrip->getAudioOutputLatency());
+        pingStats.insert(
+            QLatin1String("clientBufferLatency"),
+            m_jackTrip.isNull() ? 0 : ((int)(10 * m_jackTrip->getLatency())) / 10.0);
         emit updateNetworkStats(pingStats);
     }
 
@@ -241,7 +255,7 @@ void VsDevice::updateState(const QString& serverId)
     };
     QJsonDocument request = QJsonDocument(json);
     QNetworkReply* reply  = m_api->updateDevice(m_appID, request.toJson());
-    connect(reply, &QNetworkReply::finished, this, [=]() {
+    connect(reply, &QNetworkReply::finished, this, [reply]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
         }
@@ -267,7 +281,7 @@ void VsDevice::sendLevels()
 
     QJsonDocument request = QJsonDocument(json);
     QNetworkReply* reply  = m_api->updateDevice(m_appID, request.toJson());
-    connect(reply, &QNetworkReply::finished, this, [=]() {
+    connect(reply, &QNetworkReply::finished, this, [reply]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
         }
@@ -549,7 +563,7 @@ void VsDevice::registerJTAsDevice()
     QJsonDocument request = QJsonDocument(json);
 
     QNetworkReply* reply = m_api->postDevice(request.toJson());
-    connect(reply, &QNetworkReply::finished, this, [=]() {
+    connect(reply, &QNetworkReply::finished, this, [this, reply]() {
         if (reply->error() != QNetworkReply::NoError) {
             std::cout << "Error: " << reply->errorString().toStdString() << std::endl;
             reply->deleteLater();
